/*
 * Copyright (c) 2011, Florian Zeitz <florob@babelmonkeys.de>
 *
 * http://cgit.babelmonkeys.de/cgit.cgi/mpdbot/
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice is present in all copies.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#import <ObjFW/ObjFW.h>

#import "MPDConnection.h"

@implementation MPDConnection: OFObject
- initWithHost: (OFString*)host
	  port: (uint16_t)port
{
	self = [super init];

	mpd_host = [host copy];
	mpd_port = port;

	return self;
}

- (void)dealloc
{
	[sock release];
	[mpd_host release];

	[super dealloc];
}

- (void)connect
{
	int64_t pause = 1;
	while(1) {
		@try {
			[sock release];
			sock = [[OFTCPSocket alloc] init];
			[sock connectToHost: mpd_host
				       port: mpd_port];
			[self response];
			return;
		} @catch (id e) {
			[of_stderr writeFormat: @"Connection failed, retrying"
				@" in %" PRIi64 @" seconds\n", pause];
			[OFThread sleepForTimeInterval: pause];
			if (pause < 120)
				pause *= 2;
			[e release];
		}
	}
}

- (void)send: (OFString*)message
{
	[sock writeLine: message];
}

- (void)idle
{
	[sock writeLine: @"idle player"];
	[self response];
}

- (OFMutableDictionary*)response
{
	OFString *answer;
	OFMutableDictionary *response = [OFMutableDictionary dictionary];
	while ((answer = [sock readLine]) && ![answer hasPrefix: @"OK"]) {
		size_t index;
		index = [answer indexOfFirstOccurrenceOfString: @":"];
		if (index == OF_INVALID_INDEX)
			continue;
		[response setObject: [answer substringWithRange:
		    of_range(index + 2, answer.length - (index + 2))]
			     forKey: [answer
		    substringWithRange: of_range(0, index)]];
	}

	return response;
}
@end
